"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/lib/__init__.py $
$Id: __init__.py 29247 2007-01-29 22:12:28Z dbinger $
"""
from qpy.compile import compile_qpy_files
compile_qpy_files(__path__[0])
