"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/gallery.py $
$Id: gallery.py 29688 2007-03-29 13:46:15Z dbinger $
"""
from dulcinea.attachable import Attachable
from qp.lib.spec import Specified
from qp.pub.common import get_publisher
from durus.persistent import PersistentObject

def get_gallery():
    return get_publisher().get_connection().get_root().get('gallery')

class Gallery (PersistentObject, Specified, Attachable):

    def get_allowed_mime_types(self, user=None):
        return ['image/jpeg',
                'image/gif',
                'image/png']

