"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/session.py $
$Id: session.py 29688 2007-03-29 13:46:15Z dbinger $
"""
from dulcinea.attachable import Attachable
from dulcinea.user import DulcineaUser
from qp.lib.spec import Specified
from qp.pub.session import Session


class DulcineaSession (Session, Specified, Attachable):

    effective_user_is = DulcineaUser
    owner_is = DulcineaUser

    def __init__(self):
        Session.__init__(self)
        Attachable.__init__(self)

    def clear_app_state(self):
        """Override to clear any session data
        """
