"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/test/utest_table.py $
$Id: utest_table.py 27828 2006-01-17 21:04:08Z dbinger $
"""
from dulcinea.ui.table import Table
from dulcinea.common import CommonTest

class Test (CommonTest):

    def a(self):
        table = Table()
        table.tbody_id = 'test_id'
        table.render()
        table.column(a=1)
        table.column(b=2)
        table.render()
        for x in range(1000):
            table.row(a=x, b=x)
        table.render()

if __name__ == '__main__':
    Test()
