"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/property/errors.py $
$Id: errors.py 28932 2006-10-12 20:29:05Z dbinger $
"""

class PropertyError (Exception):
    """Base class for all property-related exceptions."""

class PropertyTypeError (PropertyError):
    """Raised on violation of the property type system"""

class ConstraintError (PropertyError):
    """
    Raised when attempting to set a property value in violation
    of the property's constraint.
    """


