"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/property/__init__.py $
$Id: __init__.py 29368 2007-02-12 16:25:16Z rmasse $
"""
from qp.pub.common import get_publisher

def get_property_db():
    return get_publisher().get_connection().get_root().get('property_db')
