"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/__init__.py $
$Id: __init__.py 30614 2008-03-17 15:47:42Z dbinger $
"""
__version__ = "0.18"

try:
    import dulcinea.common
except ImportError:
    print("Dulcinea %s" % __version__)
